#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "mpt1327.h"
#include "icom.h"

#define MPT1327_POLYNOM 0x6815

//#define BASEFREQ 162.0625   /* Base freq for channel to freq calcs */
                       /* Channel 0 (bin 0000000000) has used in CHAN */

//#define	OFFSET 1

                       /*  229 for Spain RadioRed or band A networks */
                       /*  231 for some Spain band B networks */
                       /*  for more information about spain systems see CNAF UN-30 */

                       /*  UK MPT1347 Band III Sub-band I 170.7875 Sub-band II 201.2  - NOT TESTED */

                       /*  Other networks, check local trunking bands plan */

//#define STEP 0.0125    /* Band plan step size in Mhz */

//#define CC 229.087500        /* Control Channel, only needed with same scanner for CC & TRACKING */

#define TARGET_PREFIX 0xffff     /* Target Prefix - 0xffff to track all */ 
#define TARGET_IDENT 0xffff     /* Target Ident to Track - 0xffff to track all idents within prefix */

#define VERBOSE 0    /* Verbose mode - show extra information for debugging */

double baseFreq;
double offset;
double step;

void mpt1327_setup(void)
{
	char CurrDir[MAX_PATH];
 
	GetCurrentDirectory(MAX_PATH,CurrDir);
	strcat(CurrDir,"\\trunkito.ini");
    
	baseFreq = ((double)GetPrivateProfileInt("MPT1327","BASE",1620625,CurrDir) / 10000.0);
	offset = (double)GetPrivateProfileInt("MPT1327","OFFSET",1,CurrDir);
	step = ((double)GetPrivateProfileInt("MPT1327","STEP",0125,CurrDir) / 10000.0);
}

static int
getbit(int n, unsigned short *codewords)
{
    unsigned short mask;
    unsigned short codeword;

    codeword = codewords[n/16];
    mask = 1<<(15-(n%16));

    return (codeword & mask)?1:0;
}


static unsigned short
mpt1327_fcs(mpt1327_t *m)
{
    int n, bit;
    unsigned short syndrome = 0, parity = 0;

    for (n = 0; n < 64; n++) {
	bit = getbit(n, m->codeword);
	parity ^= bit;
	if (n == 62) bit ^= 1;
	if (n < 63) {
	    syndrome <<= 1;
	    if (bit ^ ((syndrome & 0x8000)?1:0))
		syndrome ^= 0x6815;
	}
    }

    syndrome &= 0x7fff;
    if (parity)
	syndrome |= 0x8000;
    
    return syndrome;
}


void
mpt1327(unsigned char bit, mpt1327_t *m)
{
    unsigned short para;

    unsigned short channel;
    double channel2;         
    unsigned short prefix;        
    unsigned short ident1;
    unsigned short cat;
    unsigned short type;
    unsigned short func;

    
    /*
     * Shift in the least significant bit into the 64 bit frame.
     */
    m->codeword[0] <<= 1;
    m->codeword[0] += (m->codeword[1] & 0x8000)?1:0;
    m->codeword[1] <<= 1;
    m->codeword[1] += (m->codeword[2] & 0x8000)?1:0;
    m->codeword[2] <<= 1;
    m->codeword[2] += (m->codeword[3] & 0x8000)?1:0;
    m->codeword[3] <<= 1;
    if (bit) m->codeword[3] += 1;
    m->cnt++;

    switch (m->state) {
    case 0:
	if (((m->codeword[3] & 0xffff) == MPT1327_SYNC) ||
	    ((m->codeword[3] & 0xffff) == MPT1327_SYNT)) {
	    if (mpt1327_fcs(m) == 0) {
#if 1
		switch (m->codeword[0] & 0x7fff) { /* SYS */
		case 0x0509:
		    printf("BUDBIL ");
		    break;
		case 0x0781:
		    printf("SKANSKA ");
		    break;
		case 0x2491:
		    printf("MPT Comm ");
		    break;
		default:
		    printf("SYS 0x%4.4x ", m->codeword[0] & 0x7fff);
		}

		fflush(stdout);
#endif
	    }
	    m->state = 1;
	    m->cnt = 0;
	}
	break;
    case 1:
	if (m->cnt == 64) {
	    if (mpt1327_fcs(m) == 0) {
#if 0
		if (m->codeword[0] & 0x8000) {
		    printf("A ");                     /* Address Codeword */
		} else {
		    printf("D ");                     /* Data Codeword */
		}
#endif

                cat = ((m->codeword[1]>>7) & 0x7);
                type = ((m->codeword[1]>>5) & 0x3);
                func = ((m->codeword[1]>>2) & 0x7);

             
#if VERBOSE

			printf("CAT: 0x%1.1x ", (m->codeword[1]>>7) & 0x7);
			printf("TYPE: 0x%1.1x ", (m->codeword[1]>>5) & 0x3);
			printf("FUNC: 0x%1.1x ", (m->codeword[1]>>2) & 0x7);
			printf("\n");
#endif

                if (cat == 0x0) {
                  if (type == 0x0) printf("ALOHA ");
                  if (type == 0x1) {
                   printf("ACK");
                   if (func == 0x0) printf("K General ");
                   if (func == 0x1) printf("I Intermediate ");
                   if (func == 0x2) printf("Q Call Queued");
                   if (func == 0x3) printf("X Mensage rejected ");
                   if (func == 0x4) printf("V Called unit unavailable ");
                   if (func == 0x5) printf("E ** EMERGENCY CALL **");
                   if (func == 0x6) printf("T Try on given address ");
                   if (func == 0x7) printf("B Call Back or Negative ");

                   prefix = ((m->codeword[0] & 0x7f00) >> 8);
                   printf(" Prefix: 0x%x",prefix);

                   ident1 = ((m->codeword[0] << 5 | m->codeword[1] >> 11) & 0x1fff);
                   printf(" Ident1: 0x%x ",ident1);
                  
                  }
                  
                  if (type == 0x2) printf("REQ / AHOY ");
                  if (type == 0x3) printf("MISC ");

                  printf("\n");
                }

                if (cat == 0x1 && type == 0x0) {
                 printf ("Single Address Message \n");
                }

                if (cat == 0x1 && type == 0x1) {
                 printf ("Sort Data Message \n");
                }

               
                if (cat == 0x0 && type == 0x3 && func == 0x1) {
                 printf ("Call Maintenance Message - MAINT \n");
                }

               
                if (cat == 0x0 && type == 0x3 && func == 0x3) {
                 printf ("Move to Control Channel - MOVE \n");
                }

		if (m->codeword[1] & (1<<10)) {
#if 1
		    para = m->codeword[2];
		    para += (m->codeword[1] & 0x03) << 16;

		    /* Endast nedkopplingen */
		    if ((para & 0xaaa) == 0xaaa) {
			time_t t = time(NULL);
			channel = (m->codeword[0]>>5 & 0x2ff);
                        channel2 = (baseFreq+((channel+offset-2)*step));

                                        /* Clear Down Allocated Traffic Channel */
                         
			printf("CLEAR CHAN: 0x%x Freq:%f %s", channel, channel2, ctime(&t));

                                        /* We back to Control Channel */
                        //printf("[END ");
                        //setfreq(CC);

                        printf("\n");


#if 0
			printf("CAT: 0x%1.1x ", (m->codeword[1]>>7) & 0x7);
			printf("TYPE: 0x%1.1x ", (m->codeword[1]>>5) & 0x3);
			printf("FUNC: 0x%1.1x ", (m->codeword[1]>>2) & 0x7);
			printf("PARA: 0x%5.5x ", para);
			printf("CHAN: 0x%x", m->codeword[0]>>5 & 0x2ff);
			printf("\n");
#endif
		    }
#endif
		} else {
		    printf("GTC ");
		    channel = ( (m->codeword[1] << 1 | m->codeword[2] >> 15) & 0x3ff);
		    printf("Channel 0x%x ", channel);
                    channel2 = (baseFreq+((channel+offset-2)*step));
                    printf ("Freq: %f Mhz ", channel2);

#if VERBOSE
                    printbins(m->codeword[0]);
                    printbins(m->codeword[1]);
                    printbins(m->codeword[2]);
                    printbins(m->codeword[3]);
#endif

                    prefix = ((m->codeword[0] & 0x7f00) >> 8);
                    printf(" Prefix: 0x%x",prefix);

                    ident1 = ((m->codeword[0] << 5 | m->codeword[1] >> 11) & 0x1fff);
                    printf(" Ident1: 0x%x ",ident1);

                    if (m->codeword[1] & 0x200) {
                      printf("Data ");                                   /* Data Call */
                    } else {
                      printf("Voice ");                                  /* Voice Call */
                      if ((prefix & TARGET_PREFIX) == prefix) {
                       if ((ident1 & TARGET_IDENT) == ident1) {   
                         //printf("[");
                         //setfreq(channel2);
                       }
                      }
                    }


#if  0
                    printf("RAW DATA 0x%x ", m->codeword[0]);
                    printf("%x ", m->codeword[1]);
                    printf("%x ", m->codeword[2]);
                    printf("[%x]", m->codeword[3]);
#endif
                    printf("\n");


		}

	    }
	    m->state = 0;
	    m->cnt = 0;
	}
	break;
    default:
	m->state = 0;
    }
}

void printbins(unsigned short x)
{
  int n;
 
 printf(" ");

  for (n=0; n<16; n++)
  {
      if ((x & 0x8000) !=0) {
       printf("1");
      }
      else { printf("0");}
      
      if (n==7) { printf(" ");}
      x = x<<1;
  }
}
