#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "mpt1327.h"
#include "icom.h"

#define MPT1327_POLYNOM 0x6815


#define TARGET_PREFIX 0xffff     /* Target Prefix - 0xffff to track all */ 
#define TARGET_IDENT 0xffff     /* Target Ident to Track - 0xffff to track all idents within prefix */

#define VERBOSE 0    /* Verbose mode - show extra information for debugging */

double baseFreq;
double offset;
double step;
double control;

int sysprint=0;
char sys[7]="0xffff";
unsigned short currentsys=0xffff;                   
char sysname[255];
int removemsg=FALSE;
char ptext[255];

FILE *log = NULL;

void pprint (char *msg){
unsigned  int idx;

  if (!memcmp(msg,"ALO",3)||!memcmp(msg,"REQ",3)||!memcmp(msg,"MIS",3)){
    if (removemsg){
      printf("          ");
      for(idx=0;idx<10;idx++) printf("%c",0x08);
      removemsg=FALSE;
    }
    else {
      printf("%s       ",msg);
      for(idx=0;idx<(strlen(msg)+7);idx++) printf("%c",0x08);
      removemsg=TRUE;
    }
  }
  else{
    removemsg=FALSE;
    if (!memcmp(msg,"SYS",3) && !sysprint){
      sysprint=1;
      printf(msg);
      fprintf(log,msg);
    }
    else
      if (memcmp(msg,"SYS",3)){
        printf("%s\n",msg);
        fprintf(log,"%s\n",msg);
        sysprint=0;
      }
  }
  fflush(log);
}


void mpt1327_setup(void)
{
	char CurrDir[MAX_PATH];
 
 	GetCurrentDirectory(MAX_PATH,CurrDir);
	strcat(CurrDir,"\\tronquito.ini");
    
    baseFreq = ((double)GetPrivateProfileInt(sys,"BASE",2290000,CurrDir) / 10000.0);
	offset = (double)GetPrivateProfileInt(sys,"OFFSET",2,CurrDir);
	step = ((double)GetPrivateProfileInt(sys,"STEP",125,CurrDir) / 10000.0);
	control = ((double)GetPrivateProfileInt(sys,"CC",2290000,CurrDir) / 10000.0);
	GetPrivateProfileString(sys,"NAME","Unknown network",sysname,sizeof(sysname),CurrDir);
	
	if (!log){
      log=fopen("log.txt","at");
      time_t t = time(NULL);
      sprintf(ptext,"\n  === TRONKITO - MPT1327 Trunktracker ===\t%s",ctime(&t));
      pprint(ptext);
      }
}

static int
getbit(int n, unsigned short *codewords)
{
    unsigned short mask;
    unsigned short codeword;

    codeword = codewords[n/16];
    mask = 1<<(15-(n%16));

    return (codeword & mask)?1:0;
}


static unsigned short
mpt1327_fcs(mpt1327_t *m)
{
    int n, bit;
    unsigned short syndrome = 0, parity = 0;

    for (n = 0; n < 64; n++) {
	bit = getbit(n, m->codeword);
	parity ^= bit;
	if (n == 62) bit ^= 1;
	if (n < 63) {
	    syndrome <<= 1;
	    if (bit ^ ((syndrome & 0x8000)?1:0))
		syndrome ^= 0x6815;
	}
    }

    syndrome &= 0x7fff;
    if (parity)
	syndrome |= 0x8000;
    
    return syndrome;
}


void
mpt1327(unsigned char bit, mpt1327_t *m)
{
    unsigned short para;
    unsigned char beep=7;
    unsigned short channel;
    unsigned short channelc;
    double channel2;         
    unsigned short prefix;        
    unsigned short ident1;
    unsigned short ident2;
    unsigned short cat;
    unsigned short type;
    unsigned short func;

    
    /*
     * Shift in the least significant bit into the 64 bit frame.
     */
    m->codeword[0] <<= 1;
    m->codeword[0] += (m->codeword[1] & 0x8000)?1:0;
    m->codeword[1] <<= 1;
    m->codeword[1] += (m->codeword[2] & 0x8000)?1:0;
    m->codeword[2] <<= 1;
    m->codeword[2] += (m->codeword[3] & 0x8000)?1:0;
    m->codeword[3] <<= 1;
    if (bit) m->codeword[3] += 1;
    m->cnt++;

    switch (m->state) {
    case 0:
	if (((m->codeword[3] & 0xffff) == MPT1327_SYNC) ||
	    ((m->codeword[3] & 0xffff) == MPT1327_SYNT)) {
	    if (mpt1327_fcs(m) == 0) {
#if 1

	    if (currentsys!=(m->codeword[0] & 0x7fff)){
	      currentsys=m->codeword[0] & 0x7fff;
	      sprintf(sys,"0x%4.4x",currentsys);
	      mpt1327_setup();
	      pprint("**************************   New Trunking Network   **************************");
        
          sprintf(ptext,"  SYS 0x%4.4x  | %s |  Control Ch.: %.4f MHz\n******************************************************************************\n", m->codeword[0] & 0x7fff, sysname, control);
          pprint(ptext);
          }
        //sprintf(ptext,"SYS 0x%4.4x ", m->codeword[0] & 0x7fff);
        //pprint(ptext);

#endif
	    }
	    m->state = 1;
	    m->cnt = 0;
	}
	break;
    case 1:
	if (m->cnt == 64) {
	    if (mpt1327_fcs(m) == 0) {
#if 0
		if (m->codeword[0] & 0x8000) {
		    printf("A ");                     /* Address Codeword */
		} else {
		    printf("D ");                     /* Data Codeword */
		}
#endif

                cat = ((m->codeword[1]>>7) & 0x7);
                type = ((m->codeword[1]>>5) & 0x3);
                func = ((m->codeword[1]>>2) & 0x7);

             
#if VERBOSE

			printf("CAT: 0x%1.1x ", (m->codeword[1]>>7) & 0x7);
			printf("TYPE: 0x%1.1x ", (m->codeword[1]>>5) & 0x3);
			printf("FUNC: 0x%1.1x ", (m->codeword[1]>>2) & 0x7);
			printf("\n");
#endif

                if (cat == 0x0) {
                  if (type == 0x0) pprint("ALOHA");
                  if (type == 0x1) {
                    sprintf(ptext,"- ACK");
                    if (func == 0x0) strcat(ptext,"K |             General            |");
                    if (func == 0x1) strcat(ptext,"I |          Intermediate          |");
                    if (func == 0x2) strcat(ptext,"Q |           Call queued          |");
                    if (func == 0x3) strcat(ptext,"X |        Message rejected        |");
                    if (func == 0x4) strcat(ptext,"V |    Called unit unavailable     |");
                    if (func == 0x5) strcat(ptext,"E |        ** EMERGENCY **         |");
                    if (func == 0x6) strcat(ptext,"T |      Try on given adress       |");
                    if (func == 0x7) strcat(ptext,"B |    Call back / Negative ACK    |");

#
                    prefix = ((m->codeword[0] & 0x7f00) >> 8);
                    ident1 = ((m->codeword[0] << 5 | m->codeword[1] >> 11) & 0x1fff);
                    ident2 = ((m->codeword[1] << 11 | m->codeword[2] >> 5) & 0x1fff);
                    sprintf(ptext,"%s     (%3d)  From: %4d   To: %4d",ptext,prefix,ident2,ident1);
                    pprint(ptext);
                  }
                  
                  if (type == 0x2) pprint("REQ / AHOY");
                  if (type == 0x3) pprint("MISC");
                }

                if (cat == 0x1 && type == 0x0) {
                 pprint ("\tSingle adress message\n");
                }

                if (cat == 0x1 && type == 0x1) {
                 pprint ("\tShort data message\n");
                }

               
                if (cat == 0x0 && type == 0x3 && func == 0x1) {
                channel = ((m->codeword[1] << 8 | m->codeword[2] >> 8) & 0x3ff);
                channel2 = (baseFreq+((channel+offset-2)*step));
                printf ("- MAINT |      Call maintenance      |   (This Ch: %d Freq: %.4f MHz\n)", channel, channel2);
                }

    # if VERBOSE
                    printf("(0)");
                    printbins(m->codeword[0]);
                    printf("(1)");
                    printbins(m->codeword[1]);
                    printf("(2)");
                    printbins(m->codeword[2]);
                    printf("(3)");
                    printbins(m->codeword[3]);
#endif          
                if (cat == 0x0 && type == 0x3 && func == 0x3) {
                channel = ((m->codeword[1] << 8 | m->codeword[2] >> 8) & 0x3ff);
                channel2 = (baseFreq+((channel+offset-2)*step));
                 printf("%c",beep);
                 printf ("- MOVE |     GO TO CONTROL CHANNEL!     |      ->     Ch:%3d Freq:%.4f MHz\n",channel,channel2);
                }

		if (m->codeword[1] & (1<<10)) {
#if 1
		    para = m->codeword[2];
		    para += (m->codeword[1] & 0x03) << 16;

		    /* Endast nedkopplingen */
		    if ((para & 0xaaa) == 0xaaa) {
			time_t t = time(NULL);
			channel = (m->codeword[0]>>5 & 0x2ff);
			channelc = ((m->codeword[0] << 5 | m->codeword[1] >> 11) & 0x3ff);
            channel2 = (baseFreq+((channel+offset-2)*step));

                                        /* Clear Down Allocated Traffic Channel */
                         
			sprintf(ptext,"- CLEAR |     Clear down    |    Ch: %d  Freq: %.4f   -   Control Ch.: %d\n\t\t%s", channel, channel2,channelc, ctime(&t));
			pprint(ptext);

                                        /* We back to Control Channel */
                        //printf("[END ");
                        //setfreq(control);

#if 0
			printf("CAT: 0x%1.1x ", (m->codeword[1]>>7) & 0x7);
			printf("TYPE: 0x%1.1x ", (m->codeword[1]>>5) & 0x3);
			printf("FUNC: 0x%1.1x ", (m->codeword[1]>>2) & 0x7);
			printf("PARA: 0x%5.5x ", para);
			printf("CANAL: 0x%d", m->codeword[0]>>5 & 0x2ff);
			printf("\n");
#endif
		    }
#endif
		} else {
		    sprintf(ptext,"-  GTC | Call type:");
			if (m->codeword[1] & 0x200) {
                    strcat(ptext," data  | ");                                   /* Data Call */
                  } else {
                    strcat(ptext," VOICE | ");                                   /* Voice Call */
                    printf("%c",beep);                                 
                    if ((prefix & TARGET_PREFIX) == prefix) {
                    if ((ident1 & TARGET_IDENT) == ident1) {   
                         //printf("[");
                         //setfreq(channel2);
                     }
                   }
               }	    
            
                    prefix = ((m->codeword[0] & 0x7f00) >> 8);
                    ident1 = ((m->codeword[0] << 5 | m->codeword[1] >> 11) & 0x1fff);
                    ident2 = ((m->codeword[2] >>2) & 0x1fff);
                    sprintf(ptext,"%s(%3d) From:%4d To:%4d |", ptext,prefix, ident2,ident1);
                    
                    channel = ( (m->codeword[1] << 1 | m->codeword[2] >> 15) & 0x3ff);
		            sprintf(ptext,"%s Ch:%3d ", ptext, channel);
                    channel2 = (baseFreq+((channel+offset-2)*step));
                    sprintf (ptext,"%sFreq:%.4f Mhz", ptext, channel2);
                    if (!(m->codeword[1] & 0x200)) {
                        time_t t = time(NULL);
                        sprintf (ptext,"%s \t\t%s", ptext, ctime(&t));
                    }

#if VERBOSE
                    printf("(0)");
                    printbins(m->codeword[0]);
                    printf("(1)");
                    printbins(m->codeword[1]);
                    printf("(2)");
                    printbins(m->codeword[2]);
                    printf("(3)");
                    printbins(m->codeword[3]);
#endif

     



#if  0
                    printf("RAW DATA 0x%x ", m->codeword[0]);
                    printf("%x ", m->codeword[1]);
                    printf("%x ", m->codeword[2]);
                    printf("[%x]", m->codeword[3]);
#endif
                    pprint(ptext);


		}

	    }
	    m->state = 0;
	    m->cnt = 0;
	}
	break;
    default:
	m->state = 0;
    }
}

void printbins(unsigned short x)
{
  int n;
 
 printf(" ");

  for (n=0; n<16; n++)
  {
      if ((x & 0x8000) !=0) {
       printf("1");
      }
      else { printf("0");}
      
      if (n==7) { printf(" ");}
      x = x<<1;
  }
}
